import requests
from requests.auth import HTTPBasicAuth
from wristband.exceptions import (
    AuthenticationError,
    AuthorizationError,
    BadRequestError,
)

def get_token(application_vanity_domain, client_id, client_secret):
    # Construct the URL
    url = f'https://{application_vanity_domain}/api/v1/oauth2/token'

    # Headers to indicate the type of data being sent
    headers = {
        'Content-Type': 'application/x-www-form-urlencoded',
    }

    # Data payload
    payload = {
        'grant_type': 'client_credentials',
    }

    try:
        # Make the request using HTTP Basic Authentication
        response = requests.post(
            url,
            headers=headers,
            data=payload,
            auth=HTTPBasicAuth(client_id, client_secret)
        )
        response.raise_for_status()
    except requests.exceptions.HTTPError as http_err:
        if response.status_code == 401:
            raise AuthenticationError(
                "Client credentials are not valid - please rerun script & enter valid credentials"
            ) from http_err
        elif response.status_code == 400:
            raise BadRequestError(
                "Application vanity domain is not valid - please rerun script & enter valid credentials"
            ) from http_err
        else:
            raise AuthorizationError(
                f"HTTP error occurred: {http_err}"
            ) from http_err
    except requests.exceptions.RequestException as err:
        raise BadRequestError(
            "Domain name is not valid - please rerun script & enter a valid domain name"
        ) from err

    # Return the access token from the response JSON
    return response.json().get('access_token')


import requests
from requests.auth import HTTPBasicAuth
import pandas as pd
from wristband.exceptions import AuthenticationError, AuthorizationError, BadRequestError
import math
from datetime import datetime
import os

class UsersService:
    def __init__(self, token=None, application_vanity_domain=None, tenant_id=None, identity_provider_name=None):
        self.token = token
        self.application_vanity_domain = application_vanity_domain
        self.tenant_id = tenant_id
        self.identity_provider_name = identity_provider_name

        self.input_file_name = 'input_users.csv'
        self.allowed_user_fields  = [
            'username',
            'password',
            'email',
            'emailVerified',
            'externalId',
            'fullName',
            'givenName',
            'familyName',
            'middleName',
            'honorificPrefix',
            'honorificSuffix',
            'nickname',
            'displayName',
            'pictureUrl',
            'gender',
            'birthdate',
            'phoneNumber',
            'preferredLanguage',
            'locale',
            'timeZone'
        ]

    def upload_users_csv(
        self,
        invite_users = True
    ): 
        # Initialize log
        logs = []

        # Get users
        users = self.get_input_users_from_csv()
        for user in users:
            create_user_response = self.create_user(user_fields=user)
            status_code = create_user_response.status_code
            response_json = create_user_response.json()
            
            log = {
                'email': user.get('email'),
                'status_code': status_code,
            }

            if status_code != 201:
                log['message'] = response_json
            elif invite_users:
                invite_exisiting_user_response = self.invite_existing_user(user_id=response_json.get('id'))
                log['message'] = invite_exisiting_user_response.json()['existingUserInvitationRequest']['status']

            logs.append(log)

        timestamp = datetime.now().strftime('%Y-%m-%d %H:%M:%S')
        if not os.path.exists('logs'):
            os.makedirs('logs')
        pd.DataFrame(logs).to_csv(f'logs/{timestamp}.csv', index=False)
        return logs
    
    def create_user(
        self,
        user_fields: dict
    ):
        """
        API Docs - https://docs.wristband.dev/reference/createuserv1
        """
        if not self.token or not self.application_vanity_domain or not self.tenant_id or not self.identity_provider_name:
            raise BadRequestError("Service is not properly initialized with required credentials.")

        # Construct the URL
        url = f'https://{self.application_vanity_domain}/api/v1/users'

        # Headers to indicate the type of data being sent
        headers = {
            'content-type': 'application/json',
            'accept': 'application/json',
            'authorization': f'Bearer {self.token}',
        }

        request_body = {
            'tenantId': self.tenant_id,
            'identityProviderName': self.identity_provider_name,
        }

        # Check if the user_fields contain any invalid fields
        for field in user_fields.keys():
            if field not in self.allowed_user_fields:
                raise BadRequestError(f"Invalid field: {field}")
            elif isinstance(user_fields[field], float) and math.isnan(user_fields[field]):
                continue  # Skip adding fields with NaN values
            else:
                request_body[field] = user_fields[field]

        # Perform the GET request
        response = requests.post(url, headers=headers, json=request_body)

        if response.status_code == 404:
            raise BadRequestError("ApplicationId is not valid - please rerun script & enter a valid applicationId")
        elif response.status_code == 403:
            raise AuthorizationError("Client is not authorized to perform the user export - please make sure that the client has the appropriate permissions assigned to it and then rerun script")

        # Return response
        return response
    
    def invite_existing_user(
        self,
        user_id: str
    ):
        """
        API Docs - https://docs.wristband.dev/reference/inviteexistinguserv1
        """
        if not self.token or not self.application_vanity_domain or not self.tenant_id or not self.identity_provider_name:
            raise BadRequestError("Service is not properly initialized with required credentials.")

        # Construct the URL
        url = f'https://{self.application_vanity_domain}/api/v1/existing-user-invitation/invite-user'

        # Headers to indicate the type of data being sent
        headers = {
            'content-type': 'application/json',
            'accept': 'application/json',
            'authorization': f'Bearer {self.token}',
        }

        # Construct the request body
        request_body = {
            'userId': user_id
        }

        # Perform the POST request with JSON payload
        response = requests.post(url, headers=headers, json=request_body)

        if response.status_code == 404:
            raise BadRequestError("ApplicationId is not valid - please rerun script & enter a valid applicationId")
        elif response.status_code == 403:
            raise AuthorizationError("Client is not authorized to perform the user export - please make sure that the client has the appropriate permissions assigned to it and then rerun script")

        # Return response
        return response          
        
    def create_input_users_csv(self):
        # Create a DataFrame with the allowed user fields as columns
        df = pd.DataFrame(columns=self.allowed_user_fields)

        # Save the DataFrame to a CSV file
        df.to_csv(f'{self.input_file_name}.csv', index=False)

    def get_input_users_from_csv(self):
        # Read the CSV file into a DataFrame
        df = pd.read_csv(f'{self.input_file_name}.csv')

        # Convert the DataFrame to a dictionary
        user_fields = df.to_dict(orient='records')

        return user_fields


import argparse
from wristband.oauth2.token_utils import get_token
from wristband.exceptions import (
    get_non_empty_response,
    AuthenticationError,
    AuthorizationError,
    BadRequestError,
)

def main():
    # Create the parser
    parser = argparse.ArgumentParser(description="Get an OAuth2 access token.")

    # Add arguments
    parser.add_argument('--application_vanity_domain', type=str, help='Application vanity domain')
    parser.add_argument('--client_id', type=str, help='Client ID')
    parser.add_argument('--client_secret', type=str, help='Client Secret')

    # Parse arguments
    args = parser.parse_args()

    # Check if arguments were provided, otherwise prompt the user
    application_vanity_domain = args.application_vanity_domain or \
        get_non_empty_response("Enter the application vanity domain: ")
    client_id = args.client_id or \
        get_non_empty_response("Enter the client ID: ")
    client_secret = args.client_secret or \
        get_non_empty_response("Enter the client secret: ")

    try:
        token = get_token(application_vanity_domain, client_id, client_secret)
        print(f"{token}")
    except (AuthenticationError, AuthorizationError, BadRequestError) as e:
        print(f"Error: {e}")

if __name__ == "__main__":
    main()


# setup.py
from setuptools import setup, find_packages

setup(
    name='wristband',
    version='0.2.4',
    description='A package for fetching data from the Withings API',
    author='Donato Di Ferdinando',
    author_email='fddiferd@gmail.com',
    url='https://github.com/wristband-dev/wristband-python.git',
    packages=find_packages(),
    entry_points={
        'console_scripts': [
            'get_token=wristband.oauth2:main',
        ],
    },
    install_requires=[
        'requests', 'pandas'
    ],  # Add any dependencies here
)



import argparse
import sys
from wristband.users.users_utils import UsersService
from wristband.exceptions import (
    get_non_empty_response,
    AuthenticationError,
    AuthorizationError,
    BadRequestError,
)

def main():
    parser = argparse.ArgumentParser(description="Upload users from CSV")
    parser.add_argument('--token', type=str, help='Authentication token')
    parser.add_argument('--application_vanity_domain', type=str, help='Application vanity domain')
    parser.add_argument('--tenant_id', type=str, help='Tenant ID')
    parser.add_argument('--identity_provider_name', type=str, help='Identity provider name')
    parser.add_argument('--invite_users', action='store_true', help='Invite users after creation')

    # Parse arguments
    args = parser.parse_args()

    # Ensure required arguments are provided
    required_args = ['token', 'application_vanity_domain', 'tenant_id', 'identity_provider_name']
    missing_args = [arg for arg in required_args if getattr(args, arg) is None]
    if missing_args:
        print(f"Missing required arguments: {', '.join(missing_args)}")
        parser.print_help()
        sys.exit(1)

    # Initialize the service
    service = UsersService(
        token=args.token,
        application_vanity_domain=args.application_vanity_domain,
        tenant_id=args.tenant_id,
        identity_provider_name=args.identity_provider_name
    )

    # Execute the upload_user_csv method
    try:
        logs = service.upload_users_csv(invite_users=args.invite_users)
        for log in logs:
            print(log)
    except (AuthenticationError, AuthorizationError, BadRequestError) as e:
        print(f"Error: {e}")

if __name__ == "__main__":
    main()


class AuthenticationError(Exception):
    """Custom exception for authentication failures."""
    pass

class AuthorizationError(Exception):
    """Custom exception for authorization failures."""
    pass

class BadRequestError(Exception):
    """Custom exception for bad request errors."""
    pass

def get_non_empty_response(prompt):
    """Prompt the user until a non-empty response is given."""
    while True:
        response = input(prompt)
        if response.strip():
            return response
        else:
            print("This field cannot be empty. Please try again.")



from wristband.users.users_utils import UsersService

def main():
    # Initialize service without credentials
    service = UsersService()
    service.create_input_users_csv()

if __name__ == "__main__":
    main()